"""CLI commands for Lore - Thin Client version."""

from lore.cli.commands import (
    blame,
    commit,
    config,
    init,
    search,
    sync,
)

__all__ = [
    "blame",
    "commit",
    "config",
    "init",
    "search",
    "sync",
]
